@@ -129,7 +129,7 @@
 
 /* Define if your processor stores words with the most significant byte
    first (like Motorola and SPARC, unlike Intel and VAX). */
-#define WORDS_BIGENDIAN
+/* #undef WORDS_BIGENDIAN */
 
 /* VA library (libva) */
 /* #undef HAVE_VA */
